from displayio import Group
from adafruit_clue import clue
from adafruit_bitmap_font import bitmap_font
from terminalio import FONT
from adafruit_display_text import label
from adafruit_display_shapes.rect import Rect
from debug import debug
from math import sqrt

SAND_COLOR_OUT_OF_TIME = (255, 0, 0)
SAND_COLOR_TIME_LEFT = (0, 255, 0)
SAND_COLOR_TIME_USED = (120, 120, 255)

class Gui:
    __root = None
    __font = None
    __font_scale = 1
    __curr_bg_color = None
    __curr_fg_color = None
    __bg_rect = None
    __bg_rect_sandtimer = None
    __bg_rect_group = None
    __bg_rect_sandtimer_group = None
    __title = None
    __sub_title = None
    __turn_time = None
    __total_time = None

    def __init__(self) -> None:
        raise Exception('Do not initialize this class')

    def load_fonts():
            debug('loading font')
            Gui.__font = bitmap_font.load_font("/fonts/Arial-Bold-24.bdf")
            glyphs = b'0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ()-%'
            Gui.__font.load_glyphs(glyphs)

    def __create_root():
        if Gui.__root == None:
            debug('loading root group')
            Gui.__root = Group()
            Gui.__bg_rect = Rect(0, 0, 2, 2, fill=Gui.__curr_bg_color)
            Gui.__bg_rect_sandtimer = Rect(0, 0, 2, 2, fill=SAND_COLOR_TIME_USED)
            Gui.__bg_rect_group = Group(scale=120)
            Gui.__bg_rect_sandtimer_group = Group(scale=120)
            Gui.__bg_rect_group.append(Gui.__bg_rect)
            Gui.__bg_rect_sandtimer_group.append(Gui.__bg_rect_sandtimer)
            Gui.__root.append(Gui.__bg_rect_group)
            Gui.__root.append(Gui.__bg_rect_sandtimer_group)
            Gui.__bg_rect_sandtimer_group.y = -240

            Gui.__title = label.Label(Gui.__font, text=" "*20, color=Gui.__curr_fg_color, scale=Gui.__font_scale)
            Gui.__root.append(Gui.__title)

            Gui.__sub_title = label.Label(Gui.__font, text=" "*20, color=Gui.__curr_fg_color, scale=Gui.__font_scale)
            Gui.__root.append(Gui.__sub_title)

            Gui.__turn_time = label.Label(Gui.__font, text=" "*20, color=Gui.__curr_fg_color, scale=Gui.__font_scale)
            Gui.__root.append(Gui.__turn_time)

            Gui.__total_time = label.Label(Gui.__font, text=" "*20, color=Gui.__curr_fg_color, scale=Gui.__font_scale)
            Gui.__root.append(Gui.__total_time)

    def show_titles_only():
        if not Gui.__root:
            Gui.__create_root()
        Gui.__title.anchor_point = (0.5, 0)
        Gui.__title.anchored_position = (120, 90)
        Gui.__sub_title.anchor_point = (0.5, 0)
        Gui.__sub_title.anchored_position = (120, 130)
        Gui.__turn_time.anchor_point = (0.5, 0)
        Gui.__turn_time.anchored_position = (120, 500)
        Gui.__total_time.anchor_point = (0.5, 0)
        Gui.__total_time.anchored_position = (120, 500)
        clue.display.show(Gui.__root)

    def show_titles_and_times():
        if not Gui.__root:
            Gui.__create_root()
        Gui.__title.anchor_point = (0.5, 0)
        Gui.__title.anchored_position = (120, 10)
        Gui.__sub_title.anchor_point = (0.5, 0)
        Gui.__sub_title.anchored_position = (120, 200)
        Gui.__turn_time.anchor_point = (0.5, 0)
        Gui.__turn_time.anchored_position = (120, 90)
        Gui.__total_time.anchor_point = (0.5, 0)
        Gui.__total_time.anchored_position = (120, 130)
        clue.display.show(Gui.__root)

    def hide():
        Gui.__root = None
        Gui.__font = None
        Gui.__curr_bg_color = None
        Gui.__curr_fg_color = None
        Gui.__bg_rect = None
        Gui.__bg_rect_sandtimer = None
        Gui.__bg_rect_group = None
        Gui.__bg_rect_sandtimer_group = None
        Gui.__title = None
        Gui.__sub_title = None
        Gui.__turn_time = None
        Gui.__total_time = None
        clue.display.show(Group())

    def get_foreground(rgb_color=(0,128,255)):
        (r,g,b)=rgb_color
        hsp = sqrt(0.299 * (r * r) + 0.587 * (g * g) + 0.114 * (b * b))
        if (hsp>127.5):
            return (0,0,0)
        else:
            return (255,255,255)

    def set_background(bg_color, sandtimer_pos=-240):
        if (Gui.__bg_rect_sandtimer_group.y != sandtimer_pos):
            debug(f'Move sandtimer to {sandtimer_pos}')
            Gui.__bg_rect_sandtimer_group.y = sandtimer_pos
        if (Gui.__curr_bg_color != bg_color):
            Gui.__curr_bg_color = bg_color
            Gui.__bg_rect.fill = bg_color
            fg_color = Gui.get_foreground(bg_color)
            if (Gui.__curr_fg_color != fg_color):
                Gui.__curr_fg_color = fg_color
                Gui.__title.color = fg_color
                Gui.__sub_title.color = fg_color
                Gui.__turn_time.color = fg_color
                Gui.__total_time.color = fg_color


    def update_sandtimer_position(percent_remaining):
        y_pos = int(-240 * percent_remaining)  # -240 = all time remaining -> 0 = no time remaining
        Gui.set_background(SAND_COLOR_TIME_LEFT, sandtimer_pos=y_pos)

    def set_title(title_text, sub_title_text=""):
        sub = '({})'.format(sub_title_text) if sub_title_text else ""
        if Gui.__title.text != title_text or Gui.__sub_title.text != sub:
            debug(f'set_title: {title_text} ({sub_title_text})')
            Gui.__title.text = title_text
            Gui.__sub_title.text = sub

    def set_turn_time(text):
        if Gui.__turn_time.text != text:
            Gui.__turn_time.text = text

    def set_total_time(text):
        if Gui.__total_time.text != text:
            Gui.__total_time.text = text
